@@ -161,6 +161,8 @@ data Context = Context
   , ctxForeignSrcLang :: Maybe TH.ForeignSrcLang
     -- ^ TH.LangC by default
   , ctxEnableCpp :: Bool
+    -- ^ Compile source code to raw object.
+  , ctxRawObjectCompile :: Maybe (String -> TH.Q FilePath)
   }
 
 
@@ -172,6 +174,7 @@ instance Semigroup Context where
     , ctxOutput = ctxOutput ctx1 <|> ctxOutput ctx2
     , ctxForeignSrcLang = ctxForeignSrcLang ctx1 <|> ctxForeignSrcLang ctx2
     , ctxEnableCpp = ctxEnableCpp ctx1 || ctxEnableCpp ctx2
+    , ctxRawObjectCompile = ctxRawObjectCompile ctx1 <|> ctxRawObjectCompile ctx2
     }
 #endif
 
@@ -182,6 +185,7 @@ instance Monoid Context where
     , ctxOutput = Nothing
     , ctxForeignSrcLang = Nothing
     , ctxEnableCpp = False
+    , ctxRawObjectCompile = Nothing
     }
 
 #if !MIN_VERSION_base(4,11,0)
@@ -191,6 +195,7 @@ instance Monoid Context where
     , ctxOutput = ctxOutput ctx1 <|> ctxOutput ctx2
     , ctxForeignSrcLang = ctxForeignSrcLang ctx1 <|> ctxForeignSrcLang ctx2
     , ctxEnableCpp = ctxEnableCpp ctx1 || ctxEnableCpp ctx2
+    , ctxRawObjectCompile = ctxRawObjectCompile ctx1 <|> ctxRawObjectCompile ctx2
     }
 #endif
 
@@ -284,18 +289,12 @@ convertType purity cTypes = runMaybeT . go
           Just ty -> return ty
         hsTy <- forM cTys $ \cTys'  -> go (C.TypeSpecifier undefined cTys')
         case hsTy of
-          (a:[]) ->
-            lift [t| $(symbol) $(return a) |]
-          (a:b:[]) ->
-            lift [t| $(symbol) '($(return a),$(return b))|]
-          (a:b:c:[]) ->
-            lift [t| $(symbol) '($(return a),$(return b),$(return c))|]
-          (a:b:c:d:[]) ->
-            lift [t| $(symbol) '($(return a),$(return b),$(return c),$(return d))|]
-          (a:b:c:d:e:[]) ->
-            lift [t| $(symbol) '($(return a),$(return b),$(return c),$(return d),$(return e))|]
           [] -> fail $ "Can not find template parameters."
-          _ -> fail $ "Find too many template parameters. num = " ++ show (length hsTy)
+          (a:[]) ->
+            lift $ TH.AppT <$> symbol <*> return a
+          other ->
+            let tuple = foldl (\tuple arg -> TH.AppT tuple arg) (TH.PromotedTupleT (length other)) other
+            in lift $ TH.AppT <$> symbol <*> return tuple
       C.TypeSpecifier _specs (C.TemplateConst num) -> do
         let n = (TH.LitT (TH.NumTyLit (read num)))
         lift [t| $(return n) |]
